/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2010 EBM WebSourcing - http://www.petalslink.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.se.talend;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.se.talend.processor.AbstractExchangeProcessor;
import org.ow2.petals.se.talend.processor.InOnlyProcessor;
import org.ow2.petals.se.talend.processor.InOutProcessor;
import org.ow2.petals.se.talend.processor.RobustInOnlyProcessor;

/**
 * The class in charge of reacting to the reception of JBI messages.
 * @author Vincent Zurczak - EBM WebSourcing
 */
public class TalendJBIListener extends AbstractJBIListener {

	/**
	 * Checks the exchange's MEP and delegates the processing to the right processor.
	 * 
	 * @see AbstractJBIListener#onJBIMessage(Exchange)
	 * @see AbstractExchangeProcessor#onJbiMessage(String, Exchange)
	 * @see InOnlyProcessor#onJbiMessage(String, Exchange)
	 * @see InOutProcessor#onJbiMessage(String, Exchange)
	 * @see RobustInOnlyProcessor#onJbiMessage(String, Exchange)
	 */
	@Override
	public boolean onJBIMessage( final Exchange exchange ) {

		String logHint = "Exchange " + exchange.getExchangeId();
		Logger log = getLogger();
		if( log.isLoggable( Level.INFO ))
			log.info( logHint + " is started to be processed." );

		AbstractExchangeProcessor processorDelegate = null;
		if( exchange.isInOutPattern())
			processorDelegate = new InOutProcessor((TalendSe) getComponent());
		else if( exchange.isInOnlyPattern())
			processorDelegate = new InOnlyProcessor((TalendSe) getComponent());
		else if( exchange.isRobustInOnlyPattern())
			processorDelegate = new RobustInOnlyProcessor((TalendSe) getComponent());

		boolean result = true;
		if( processorDelegate != null ) {
			result = processorDelegate.onJbiMessage( logHint, exchange );
		}
		else {
			// InOptionalOut supports errors.
			String msg = "The MEP " + exchange.getPattern() + " is not supported in the Petals-SE-Talend component.";
			if( log.isLoggable( Level.FINE ))
				log.fine( logHint + " encountered a problem. " + msg );

			// setError => the exception must be in a standard package
			exchange.setError( new IllegalArgumentException( msg ));
		}

		return result;
	}
}
